@@ -23,223 +23,19 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Log: q931.h,v $
- * Revision 2.12  2005/11/30 13:35:26  csoutheren
- * Changed tags for Doxygen
- *
- * Revision 2.11  2004/02/19 10:46:44  rjongbloed
- * Merged OpenH323 version 1.13.1 changes.
- *
- * Revision 2.10  2003/01/07 04:39:53  robertj
- * Updated to OpenH323 v1.11.2
- *
- * Revision 2.9  2002/11/10 11:33:17  robertj
- * Updated to OpenH323 v1.10.3
- *
- * Revision 2.8  2002/09/16 02:52:34  robertj
- * Added #define so can select if #pragma interface/implementation is used on
- *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
- *
- * Revision 2.7  2002/09/04 06:01:47  robertj
- * Updated to OpenH323 v1.9.6
- *
- * Revision 2.6  2002/07/01 04:56:30  robertj
- * Updated to OpenH323 v1.9.1
- *
- * Revision 2.5  2002/02/11 09:32:12  robertj
- * Updated to openH323 v1.8.0
- *
- * Revision 2.4  2002/01/14 06:35:57  robertj
- * Updated to OpenH323 v1.7.9
- *
- * Revision 2.3  2001/10/05 00:22:13  robertj
- * Updated to PWLib 1.2.0 and OpenH323 1.7.0
- *
- * Revision 2.2  2001/08/21 01:11:12  robertj
- * Update from OpenH323
- *
- * Revision 2.1  2001/08/13 05:10:39  robertj
- * Updates from OpenH323 v1.6.0 release.
- *
- * Revision 2.0  2001/07/27 15:48:24  robertj
- * Conversion of OpenH323 to Open Phone Abstraction Library (OPAL)
- *
- * Revision 1.52  2003/03/18 05:54:14  robertj
- * Added ChannelIdentifier IE support, thanks Eize Slange
- *
- * Revision 1.51  2003/02/12 00:02:06  robertj
- * Added more Q.931 cause codes.
- * Added ability to trace text version of cause codes and IE codes.
- *
- * Revision 1.50  2002/11/25 22:40:00  robertj
- * Added another Q.850 code
- *
- * Revision 1.49  2002/11/07 03:49:49  robertj
- * Added extra "congested" Q.931 codes.
- *
- * Revision 1.48  2002/09/16 01:14:15  robertj
- * Added #define so can select if #pragma interface/implementation is used on
- *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
- *
- * Revision 1.47  2002/09/03 05:42:27  robertj
- * Normalised the multi-include header prevention ifdef/define symbol.
- * Removed redundent includes.
- *
- * Revision 1.46  2002/08/05 10:03:47  robertj
- * Cosmetic changes to normalise the usage of pragma interface/implementation.
- *
- * Revision 1.45  2002/07/05 03:13:35  robertj
- * Added copy constructor for Q.931 so makes duplicate instead of reference to IE's.
- *
- * Revision 1.44  2002/05/22 23:12:01  robertj
- * Enhanced the display of Release-Complete cause codes.
- *
- * Revision 1.43  2002/05/03 05:38:16  robertj
- * Added Q.931 Keypad IE mechanism for user indications (DTMF).
- *
- * Revision 1.42  2002/04/22 08:03:41  craigs
- * Added Q931 progress codes
- *
- * Revision 1.41  2002/04/22 07:33:02  craigs
- * Changed GetProgressIndicator to be const
- * Fixed spelling mistake in SignalType enums list
- *
- * Revision 1.40  2002/04/18 09:35:00  robertj
- * Added CallState IE processing.
- *
- * Revision 1.39  2002/03/27 06:04:42  robertj
- * Added Temporary Failure end code for connection, an application may
- *   immediately retry the call if this occurs.
- *
- * Revision 1.38  2002/02/08 00:05:14  robertj
- * Changed release complete causes so can dsitinguish between unknown
- *   cause and no cause at all.
- *
- * Revision 1.37  2002/01/07 04:25:41  robertj
- * Added support for Connected-Number Information Element, thanks Hans Verbeek
- *
- * Revision 1.36  2001/09/17 02:06:38  robertj
- * Added Redirecting Number IE to Q.931, thanks Frank Derks
- *
- * Revision 1.35  2001/09/13 02:41:32  robertj
- * Fixed call reference generation to use full range and common code, thanks Carlo Kielstra
- *
- * Revision 1.34  2001/09/12 02:01:33  robertj
- * Added "No Circuit" release complete code.
- *
- * Revision 1.33  2001/08/20 06:48:26  robertj
- * Added Q.931 function for setting bearer capabilities, allowing
- *    applications to set the data rate as they require.
- *
- * Revision 1.32  2001/08/03 14:12:07  robertj
- * Fixed value for Call State Information Element
- *
- * Revision 1.31  2001/07/24 23:40:15  craigs
- * Added ability to remove Q931 IE
- *
- * Revision 1.30  2001/06/14 06:25:13  robertj
- * Added further H.225 PDU build functions.
- * Moved some functionality from connection to PDU class.
- *
- * Revision 1.29  2001/05/30 04:38:38  robertj
- * Added BuildStatusEnquiry() Q.931 function, thanks Markus Storm
- *
- * Revision 1.28  2001/04/11 00:12:38  robertj
- * Added some enums for numbering plans and call types, thanks Markus Storm.
- *
- * Revision 1.27  2001/02/09 05:16:24  robertj
- * Added #pragma interface for GNU C++.
- *
- * Revision 1.26  2001/01/19 07:01:42  robertj
- * Added all of the Q.931 message type codes.
- *
- * Revision 1.25  2000/10/13 02:15:23  robertj
- * Added support for Progress Indicator Q.931/H.225 message.
- *
- * Revision 1.24  2000/07/09 14:53:17  robertj
- * Added facility IE to facility message.
- * Changed reference to the word "field" to be more correct IE or "Information Element"
- *
- * Revision 1.23  2000/06/21 08:07:39  robertj
- * Added cause/reason to release complete PDU, where relevent.
- *
- * Revision 1.22  2000/05/18 11:53:34  robertj
- * Changes to support doc++ documentation generation.
- *
- * Revision 1.21  2000/05/09 12:19:23  robertj
- * Added ability to get and set "distinctive ring" Q.931 functionality.
- *
- * Revision 1.20  2000/05/08 14:07:26  robertj
- * Improved the provision and detection of calling and caller numbers, aliases and hostnames.
- *
- * Revision 1.19  2000/05/06 02:17:49  robertj
- * Changed the new CallingPartyNumber code so defaults for octet3a are application dependent.
- *
- * Revision 1.18  2000/05/05 00:44:05  robertj
- * Added presentation and screening fields to Calling Party Number field, thanks Dean Anderson.
- *
- * Revision 1.17  2000/05/02 04:32:25  robertj
- * Fixed copyright notice comment.
- *
- * Revision 1.16  2000/03/21 01:07:21  robertj
- * Fixed incorrect call reference code being used in originated call.
- *
- * Revision 1.15  1999/12/23 22:43:36  robertj
- * Added calling party number field.
- *
- * Revision 1.14  1999/09/10 03:36:48  robertj
- * Added simple Q.931 Status response to Q.931 Status Enquiry
- *
- * Revision 1.13  1999/08/31 12:34:18  robertj
- * Added gatekeeper support.
- *
- * Revision 1.12  1999/08/13 06:34:38  robertj
- * Fixed problem in CallPartyNumber Q.931 encoding.
- * Added field name display to Q.931 protocol.
- *
- * Revision 1.11  1999/08/10 13:14:15  robertj
- * Added Q.931 Called Number field if have "phone number" style destination addres.
- *
- * Revision 1.10  1999/07/23 02:36:56  robertj
- * Finally found correct value for FACILITY message.
- *
- * Revision 1.9  1999/07/16 02:15:30  robertj
- * Fixed more tunneling problems.
- *
- * Revision 1.8  1999/07/09 06:09:49  robertj
- * Major implementation. An ENORMOUS amount of stuff added everywhere.
- *
- * Revision 1.7  1999/06/13 12:41:14  robertj
- * Implement logical channel transmitter.
- * Fixed H245 connect on receiving call.
- *
- * Revision 1.6  1999/06/09 05:26:20  robertj
- * Major restructuring of classes.
- *
- * Revision 1.5  1999/02/23 11:04:29  robertj
- * Added capability to make outgoing call.
- *
- * Revision 1.4  1999/01/16 11:31:46  robertj
- * Fixed name in header comment.
- *
- * Revision 1.3  1999/01/16 01:31:39  robertj
- * Major implementation.
- *
- * Revision 1.2  1999/01/02 04:00:55  robertj
- * Added higher level protocol negotiations.
- *
- * Revision 1.1  1998/12/14 09:13:41  robertj
- * Initial revision
- *
+ * $Revision: 21283 $
+ * $Author: rjongbloed $
+ * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
  */
 
-#ifndef __OPAL_Q931_H
-#define __OPAL_Q931_H
+#ifndef OPAL_H323_Q931_H
+#define OPAL_H323_Q931_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
 #endif
 
+#include <opal/buildopts.h>
 
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -283,34 +79,34 @@ class Q931 : public PObject
     Q931(const Q931 & other);
     Q931 & operator=(const Q931 & other);
 
-    void BuildFacility(int callRef, BOOL fromDest);
-    void BuildInformation(int callRef, BOOL fromDest);
+    void BuildFacility(int callRef, PBoolean fromDest);
+    void BuildInformation(int callRef, PBoolean fromDest);
     void BuildProgress(
       int callRef,
-      BOOL fromDest,
+      PBoolean fromDest,
       unsigned description,
       unsigned codingStandard = 0,
       unsigned location = 0
     );
-    void BuildNotify(int callRef, BOOL fromDest);
+    void BuildNotify(int callRef, PBoolean fromDest);
     void BuildCallProceeding(int callRef);
     void BuildSetupAcknowledge(int callRef);
     void BuildAlerting(int callRef);
     void BuildSetup(int callRef = -1);
     void BuildConnect(int callRef);
-    void BuildStatus(int callRef, BOOL fromDest);
-    void BuildStatusEnquiry(int callRef, BOOL fromDest);
-    void BuildReleaseComplete(int callRef, BOOL fromDest);
+    void BuildStatus(int callRef, PBoolean fromDest);
+    void BuildStatusEnquiry(int callRef, PBoolean fromDest);
+    void BuildReleaseComplete(int callRef, PBoolean fromDest);
 
-    BOOL Decode(const PBYTEArray & data);
-    BOOL Encode(PBYTEArray & data) const;
+    PBoolean Decode(const PBYTEArray & data);
+    PBoolean Encode(PBYTEArray & data) const;
 
     void PrintOn(ostream & strm) const;
     PString GetMessageTypeName() const;
 
     static unsigned GenerateCallReference();
     unsigned GetCallReference() const { return callReference; }
-    BOOL IsFromDestination() const { return fromDestination; }
+    PBoolean IsFromDestination() const { return fromDestination; }
     MsgTypes GetMessageType() const { return messageType; }
 
     enum InformationElementCodes {
@@ -331,9 +127,16 @@ class Q931 : public PObject
     };
     friend ostream & operator<<(ostream & strm, InformationElementCodes ie);
 
-    BOOL HasIE(InformationElementCodes ie) const;
-    PBYTEArray GetIE(InformationElementCodes ie) const;
-    void SetIE(InformationElementCodes ie, const PBYTEArray & userData);
+    PBoolean HasIE(InformationElementCodes ie) const;
+    PBYTEArray GetIE(
+      InformationElementCodes ie,
+      PINDEX idx = 0 // Index of duplicate IE entry
+    ) const;
+    void SetIE(
+      InformationElementCodes ie,
+      const PBYTEArray & userData,
+      bool append = FALSE
+    );
     void RemoveIE(InformationElementCodes ie);
 
     enum InformationTransferCapability {
@@ -352,7 +155,7 @@ class Q931 : public PObject
       unsigned userInfoLayer1 = 5   ///<  5 = Recommendations H.221 and H.242
     );
 
-    BOOL GetBearerCapabilities(
+    PBoolean GetBearerCapabilities(
       InformationTransferCapability & capability,
       unsigned & transferRate,        ///<  Number of 64k B channels
       unsigned * codingStandard = NULL,
@@ -482,7 +285,7 @@ class Q931 : public PObject
       unsigned codingStandard = 0,
       unsigned location = 0
     );
-    BOOL GetProgressIndicator(
+    PBoolean GetProgressIndicator(
       unsigned & description,
       unsigned * codingStandard = NULL,
       unsigned * location = NULL
@@ -518,7 +321,7 @@ class Q931 : public PObject
       int presentation = -1,  ///<  0 = presentation allowed, -1 = no octet3a
       int screening = -1      ///<   0 = user provided, not screened
     );
-    BOOL GetCallingPartyNumber(
+    PBoolean GetCallingPartyNumber(
       PString & number,               ///<  Number string
       unsigned * plan = NULL,         ///<  ISDN/Telephony numbering system
       unsigned * type = NULL,         ///<  Number type
@@ -533,7 +336,7 @@ class Q931 : public PObject
       unsigned plan = 1,      ///<  1 = ISDN/Telephony numbering system
       unsigned type = 0       ///<  0 = Unknown number type
     );
-    BOOL GetCalledPartyNumber(
+    PBoolean GetCalledPartyNumber(
       PString & number,       ///<  Number string
       unsigned * plan = NULL, ///<  ISDN/Telephony numbering system
       unsigned * type = NULL  ///<  Number type
@@ -547,7 +350,7 @@ class Q931 : public PObject
       int screening = -1,     ///<  0 = user provided, not screened
       int reason = -1         ///<  0 = Unknown reason , -1 = no octet 3b
     );
-    BOOL GetRedirectingNumber(
+    PBoolean GetRedirectingNumber(
       PString & number,               ///<  Number string
       unsigned * plan = NULL,         ///<  ISDN/Telephony numbering system
       unsigned * type = NULL,         ///<  Number type
@@ -567,7 +370,7 @@ class Q931 : public PObject
       int screening = -1,     ///<  0 = user provided, not screened
       int reason = -1         ///<  0 = Unknown reason , -1 = no octet 3b
     );
-    BOOL GetConnectedNumber(
+    PBoolean GetConnectedNumber(
       PString & number,               ///<  Number string
       unsigned * plan = NULL,         ///<  ISDN/Telephony numbering system
       unsigned * type = NULL,         ///<  Number type
@@ -593,7 +396,7 @@ class Q931 : public PObject
 
     /**Get the limitations to ChannelIdentification.
       */
-    BOOL GetChannelIdentification(
+    PBoolean GetChannelIdentification(
       unsigned * interfaceType = NULL,        ///<  Interface type
       unsigned * preferredOrExclusive = NULL, ///<  Channel negotiation preference
       int      * channelNumber = NULL         ///<  Channel number
@@ -601,16 +404,17 @@ class Q931 : public PObject
 
   protected:
     unsigned callReference;
-    BOOL fromDestination;
+    PBoolean fromDestination;
     unsigned protocolDiscriminator;
     MsgTypes messageType;
 
-    PDICTIONARY(InternalInformationElements, POrdinalKey, PBYTEArray);
+    PARRAY(InternalInformationElement, PBYTEArray);
+    PDICTIONARY(InternalInformationElements, POrdinalKey, InternalInformationElement);
     InternalInformationElements informationElements;
 };
 
 
-#endif // __OPAL_Q931_H
+#endif // OPAL_H323_Q931_H
 
 
 /////////////////////////////////////////////////////////////////////////////
