@@ -24,62 +24,22 @@
  *
  * The author of this code is Derek J Smithies
  *
- *  $Log: frame.h,v $
- *  Revision 1.12  2007/09/05 04:04:35  csoutheren
- *  Fixed spelling mistakes
- *
- *  Revision 1.11  2007/04/19 06:17:21  csoutheren
- *  Fixes for precompiled headers with gcc
- *
- *  Revision 1.10  2007/01/23 02:08:25  dereksmithies
- *  Handle Vnak frames correctly. handle iseqno and oseqno correctly.
- *
- *  Revision 1.9  2007/01/17 03:48:13  dereksmithies
- *  Tidy up comments, remove leaks, improve reporting of packet types.
- *
- *  Revision 1.8  2007/01/16 03:17:42  dereksmithies
- *  tidyup of comments. Remove unused variables.
- *  Guarantee that media frames are sent with a monotonically increasing timestamp
- *
- *  Revision 1.7  2006/08/09 03:46:39  dereksmithies
- *  Add ability to register to a remote Asterisk box. The iaxProcessor class is split
- *  into a callProcessor and a regProcessor class.
- *  Big thanks to Stephen Cook, (sitiveni@gmail.com) for this work.
- *
- *  Revision 1.6  2005/08/26 03:07:38  dereksmithies
- *  Change naming convention, so all class names contain the string "IAX2"
- *
- *  Revision 1.5  2005/08/25 03:26:06  dereksmithies
- *  Add patch from Adrian Sietsma to correctly set the packet timestamps under windows.
- *  Many thanks.
- *
- *  Revision 1.4  2005/08/24 04:56:25  dereksmithies
- *  Add code from Adrian Sietsma to send FullFrameTexts and FullFrameDtmfs to
- *  the remote end.  Many Thanks.
- *
- *  Revision 1.3  2005/08/24 01:38:38  dereksmithies
- *  Add encryption, iax2 style. Numerous tidy ups. Use the label iax2, not iax
- *
- *  Revision 1.2  2005/08/04 08:14:17  rjongbloed
- *  Fixed Windows build under DevStudio 2003 of IAX2 code
- *
- *  Revision 1.1  2005/07/30 07:01:32  csoutheren
- *  Added implementation of IAX2 (Inter Asterisk Exchange 2) protocol
- *  Thanks to Derek Smithies of Indranet Technologies Ltd. for
- *  writing and contributing this code
- *
- *
- *
- *
+ * $Revision: 21283 $
+ * $Author: rjongbloed $
+ * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
  */
 
-#ifndef FRAME_H
-#define FRAME_H
+#ifndef OPAL_IAX2_FRAME_H
+#define OPAL_IAX2_FRAME_H
 
 #ifndef _PTLIB_H
 #include <ptlib.h>
 #endif
 
+#include <opal/buildopts.h>
+
+#if OPAL_IAX2
+
 #include <iax2/ies.h>
 #include <iax2/remote.h>
 
@@ -1252,7 +1212,10 @@ class IAX2ActiveFrameList : public IAX2FrameList
 /////////////////////////////////////////////////////////////////////////////    
 
 
-#endif // FRAME_H
+#endif // OPAL_IAX2
+
+#endif // OPAL_IAX2_FRAME_H
+
 /* The comment below is magic for those who use emacs to edit this file. */
 /* With the comment below, the tab key does auto indent to 2 spaces.     */
 
@@ -1262,6 +1225,3 @@ class IAX2ActiveFrameList : public IAX2FrameList
  * c-basic-offset:2
  * End:
  */
-
-
-
