@@ -27,24 +27,24 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Log: iax2.h,v $
- * Revision 1.1  2005/07/30 07:01:32  csoutheren
- * Added implementation of IAX2 (Inter Asterisk Exchange 2) protocol
- * Thanks to Derek Smithies of Indranet Technologies Ltd. for
- * writing and contributing this code
- *
- *
- *
+ * $Revision: 21283 $
+ * $Author: rjongbloed $
+ * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
  */
 
-#ifndef __OPAL_IAX_H
-#define __OPAL_IAX_H
+#ifndef OPAL_IAX2_IAX2_H
+#define OPAL_IAX2_IAX2_H
+
+#include <opal/buildopts.h>
+
+#if OPAL_IAX2
 
 #include <iax2/iax2ep.h>
 #include <iax2/iax2con.h>
 
+#endif // OPAL_IAX2
 
-#endif // __OPAL_IAX_H
+#endif // OPAL_IAX2_IAX2_H
 
 
 // End of File ///////////////////////////////////////////////////////////////
