@@ -26,13 +26,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20987 $
+ * $Revision: 21283 $
  * $Author: rjongbloed $
- * $Date: 2008-09-15 12:32:43 +0000 (Mon, 15 Sep 2008) $
+ * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
  */
 
-#ifndef __OPAL_H
-#define __OPAL_H
+#ifndef OPAL_OPAL_H
+#define OPAL_OPAL_H
 
 #ifdef __cplusplus
 extern "C" {
@@ -800,7 +800,7 @@ struct OpalMessage {
 };
 #endif
 
-#endif // __OPAL_H
+#endif // OPAL_OPAL_H
 
 
 /////////////////////////////////////////////////////////////////////////////
