@@ -24,13 +24,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20801 $
+ * $Revision: 21283 $
  * $Author: rjongbloed $
- * $Date: 2008-09-01 05:05:12 +0200 (lun 01 sep 2008) $
+ * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
  */
 
-#ifndef __OPAL_CALL_H
-#define __OPAL_CALL_H
+#ifndef OPAL_OPAL_CALL_H
+#define OPAL_OPAL_CALL_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -486,7 +486,7 @@ class OpalCall : public PSafeObject
 };
 
 
-#endif // __OPAL_CALL_H
+#endif // OPAL_OPAL_CALL_H
 
 
 // End of File ///////////////////////////////////////////////////////////////
