@@ -24,13 +24,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21004 $
+ * $Revision: 21283 $
  * $Author: rjongbloed $
- * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
+ * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
  */
 
-#ifndef __OPAL_LOCAL_H
-#define __OPAL_LOCAL_H
+#ifndef OPAL_OPAL_LOCALEP_H
+#define OPAL_OPAL_LOCALEP_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -408,6 +408,7 @@ class OpalLocalMediaStream : public OpalMediaStream, public OpalMediaStreamPacin
 };
 
 
-#endif // __OPAL_LOCAL_H
+#endif // OPAL_OPAL_LOCALEP_H
+
 
 // End of File ///////////////////////////////////////////////////////////////
