@@ -23,13 +23,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20942 $
- * $Author: ms30002000 $
- * $Date: 2008-09-12 17:44:53 +0000 (Fri, 12 Sep 2008) $
+ * $Revision: 21283 $
+ * $Author: rjongbloed $
+ * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
  */
 
-#ifndef __OPAL_MEDIATYPE_H
-#define __OPAL_MEDIATYPE_H
+#ifndef OPAL_OPAL_MEDIATYPE_H
+#define OPAL_OPAL_MEDIATYPE_H
 
 #include <ptbuildopts.h>
 #include <ptlib/pfactory.h>
@@ -76,10 +76,10 @@ class OpalMediaType : public std::string     // do not make this PCaselessString
     OpalMediaType(const PString & _str)
       : std::string((const char *)_str) { }
 
-    static const char * Audio();
-    static const char * Video();
-    static const char * Fax();
-    static const char * UserInput();
+    static const OpalMediaType & Audio();
+    static const OpalMediaType & Video();
+    static const OpalMediaType & Fax();
+    static const OpalMediaType & UserInput();
 
     void PrintOn(ostream & strm) const { strm << (std::string &)*this; }
 
@@ -321,4 +321,4 @@ class OpalFaxMediaType : public OpalMediaTypeDefinition
 #endif // OPAL_T38_CAPABILITY
 
 
-#endif // __OPAL_MEDIATYPE_H
+#endif // OPAL_OPAL_MEDIATYPE_H
