@@ -26,13 +26,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21035 $
+ * $Revision: 21283 $
  * $Author: rjongbloed $
- * $Date: 2008-09-17 07:57:58 +0000 (Wed, 17 Sep 2008) $
+ * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
  */
 
-#ifndef __OPAL_RTP_H
-#define __OPAL_RTP_H
+#ifndef OPAL_RTP_RTP_H
+#define OPAL_RTP_RTP_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -1154,6 +1154,6 @@ class SecureRTP_UDP : public RTP_UDP
     OpalSecurityMode * securityParms;
 };
 
-#endif // __OPAL_RTP_H
+#endif // OPAL_RTP_RTP_H
 
 /////////////////////////////////////////////////////////////////////////////
