@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21004 $
+ * $Revision: 21158 $
  * $Author: rjongbloed $
- * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
+ * $Date: 2008-09-25 03:03:10 +0000 (Thu, 25 Sep 2008) $
  */
 
 #include <ptlib.h>
@@ -831,7 +831,7 @@ PBoolean MyManager::Initialise(PArgList & args)
   OpalMediaFormat::GetAllRegisteredMediaFormats(allMediaFormats);
   for (PINDEX i = 0; i < allMediaFormats.GetSize(); i++) {
     OpalMediaFormat mediaFormat = allMediaFormats[i];
-    if (mediaFormat.GetDefaultSessionID() == OpalMediaFormat::DefaultVideoSessionID) {
+    if (mediaFormat.GetMediaType() == OpalMediaType::Video()) {
       if (args.HasOption("video-size")) {
         PString sizeStr = args.GetOptionString("video-size");
         unsigned width, height;
