@@ -24,17 +24,15 @@
  *
  * The author of this code is Derek J Smithies
  *
- *
- *
- * $Revision: 21028 $
- * $Author: csoutheren $
- * $Date: 2008-09-16 23:32:30 +0000 (Tue, 16 Sep 2008) $
+ * $Revision: 21283 $
+ * $Author: rjongbloed $
+ * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
  */
 
 #include <ptlib.h>
 #include <opal/buildopts.h>
 
-#include <typeinfo>
+#if OPAL_IAX2
 
 #ifdef P_USE_PRAGMA
 #pragma implementation "callprocessor.h"
@@ -51,6 +49,8 @@
 #include <iax2/sound.h>
 #include <iax2/transmit.h>
 
+#include <typeinfo>
+
 #define new PNEW
 
 /********************************************************************************
@@ -1294,6 +1294,8 @@ PBoolean IAX2CallProcessor::IncomingMessageOutOfOrder(IAX2FullFrame *f)
 }
 
 
+#endif // OPAL_IAX2
+
 ////////////////////////////////////////////////////////////////////////////////
 
 /* The comment below is magic for those who use emacs to edit this file. */
