@@ -24,32 +24,30 @@
  *
  * The author of this code is Derek J Smithies
  *
- *
- *
- * $Revision: 21004 $
+ * $Revision: 21283 $
  * $Author: rjongbloed $
- * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
+ * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
  */
 
 #include <ptlib.h>
 #include <opal/buildopts.h>
 
+#if OPAL_IAX2
+
 #ifdef __GNUC__
 #pragma implementation "iax2medstrm.h"
 #endif
 
-#if OPAL_VIDEO
-#include <codec/vidcodec.h>
-#endif
+#include <iax2/iax2medstrm.h>
 
 #include <iax2/frame.h>
 #include <iax2/iax2con.h>
-#include <iax2/iax2medstrm.h>
 #include <lids/lid.h>
 #include <opal/mediastrm.h>
 #include <opal/patch.h>
 #include <ptlib/videoio.h>
 #include <rtp/rtp.h>
+#include <codec/vidcodec.h>
 
 
 #define MAX_PAYLOAD_TYPE_MISMATCHES 10
@@ -149,5 +147,7 @@ PBoolean OpalIAX2MediaStream::IsSynchronous() const
 }
 
 
+#endif // OPAL_IAX2
+
 /////////////////////////////////////////////////////////////////////////////
 
