@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20662 $
+ * $Revision: 21369 $
  * $Author: rjongbloed $
- * $Date: 2008-08-13 09:18:03 +0200 (mer 13 aoû 2008) $
+ * $Date: 2008-10-16 10:04:21 +0000 (Thu, 16 Oct 2008) $
  */
 
 #include <ptlib.h>
@@ -34,6 +34,8 @@
 #pragma implementation "lidep.cxx"
 #endif
 
+#include <opal/buildopts.h>
+
 #include <lids/lidep.h>
 
 #include <opal/manager.h>
@@ -112,16 +114,9 @@ PBoolean OpalLineEndPoint::MakeConnection(OpalCall & call,
   
   // Locate a line
   OpalLine * line = GetLine(lineName, true, terminating);
-  if (line == NULL && lineName != defaultLine) {
-    PTRACE(1,"LID EP\tMakeConnection cannot find the line \"" << lineName << '"');
-    line = GetLine(defaultLine, true, terminating);
-  }  
-  if (line == NULL){
-    PTRACE(1,"LID EP\tMakeConnection cannot find the default line " << defaultLine);
+  if (line == NULL)
     return false;
 
-  }
-
   return AddConnection(CreateConnection(call, *line, userData, number));
 }
 
@@ -147,7 +142,7 @@ OpalLineConnection * OpalLineEndPoint::CreateConnection(OpalCall & call,
                                                        void * /*userData*/,
                                                        const PString & number)
 {
-  PTRACE(3, "LID EP\tCreateConnection call = " << call << " line = " << line << " number = " << number);
+  PTRACE(3, "LID EP\tCreateConnection call = " << call << " line = \"" << line << "\", number = \"" << number << '"');
   return new OpalLineConnection(call, *this, line, number);
 }
 
@@ -352,21 +347,24 @@ OpalLine * OpalLineEndPoint::GetLine(const PString & lineName, bool enableAudio,
 {
   PWaitAndSignal mutex(linesMutex);
 
-  PTRACE(3, "LID EP\tGetLine " << lineName << ", enableAudio=" << enableAudio << ", terminating=" << terminating);
+  PTRACE(4, "LID EP\tGetLine " << lineName << ", enableAudio=" << enableAudio << ", terminating=" << terminating);
   for (OpalLineList::iterator line = lines.begin(); line != lines.end(); ++line) {
     PString lineToken = line->GetToken();
-
-    PTRACE(3, "LID EP\tGetLine name=\"" << lineToken << "\", enableAudio=" << line->IsAudioEnabled());
-
-    if ((lineName == defaultLine || lineToken == lineName) &&
-        (line->IsTerminal() == terminating) &&
-         line->IsPresent() &&
-        (!enableAudio || line->EnableAudio())) {
+    if (lineName != defaultLine && lineToken != lineName)
+      PTRACE(4, "LID EP\tNo match to line name=\"" << lineToken << "\", default=" << defaultLine);
+    else if (line->IsTerminal() != terminating)
+      PTRACE(4, "LID EP\tNo match to line name=\"" << lineToken << "\", terminating=" << line->IsTerminal());
+    else if (!line->IsPresent())
+      PTRACE(4, "LID EP\tNo match to line name=\"" << lineToken << "\", not present");
+    else if (enableAudio && (line->IsAudioEnabled() || !line->EnableAudio()))
+      PTRACE(4, "LID EP\tNo match to line name=\"" << lineToken << "\", enableAudio=" << line->IsAudioEnabled());
+    else {
       PTRACE(3, "LID EP\tGetLine found the line \"" << lineName << '"');
       return &*line;
     }  
   }
 
+  PTRACE(3, "LID EP\tGetLine could not find/enable \"" << lineName << '"');
   return NULL;
 }
 
@@ -441,7 +439,7 @@ void OpalLineEndPoint::MonitorLine(OpalLine & line)
   }
 
   // Have incoming ring, create a new LID connection and let it handle it
-  connection = CreateConnection(*call, line, NULL, PString::Empty());
+  connection = CreateConnection(*call, line, NULL, "Unknown");
   if (AddConnection(connection))
     connection->StartIncoming();
 }
@@ -458,7 +456,14 @@ OpalLineConnection::OpalLineConnection(OpalCall & call,
     line(ln)
 {
   localPartyName = ln.GetToken();
-  remotePartyNumber = number.Right(number.Find(':'));
+  remotePartyNumber = number.Right(number.FindSpan("0123456789*#,"));
+  remotePartyName = number;
+  if (remotePartyName.IsEmpty())
+    remotePartyName = "Unknown";
+  else
+    remotePartyAddress = remotePartyName + '@';
+  remotePartyAddress += GetToken();
+
   silenceDetector = new OpalLineSilenceDetector(line, (endpoint.GetManager().GetSilenceDetectParams()));
 
   minimumRingCount = 2;
@@ -555,7 +560,8 @@ PBoolean OpalLineConnection::SetConnected()
     wasOffHook = true;
   }
 
-  ownerCall.OpenSourceMediaStreams(*this, OpalMediaType::Audio());
+  if (GetMediaStream(OpalMediaType::Audio(), true) == NULL)
+    ownerCall.OpenSourceMediaStreams(*this, OpalMediaType::Audio());
   return OpalConnection::SetConnected();
 }
 
@@ -574,7 +580,7 @@ OpalMediaStream * OpalLineConnection::CreateMediaStream(const OpalMediaFormat &
                                                         unsigned sessionID,
                                                         PBoolean isSource)
 {
-  if (sessionID != OpalMediaFormat::DefaultAudioSessionID)
+  if (mediaFormat.GetMediaType() != OpalMediaType::Audio())
     return OpalConnection::CreateMediaStream(mediaFormat, sessionID, isSource);
 
   return new OpalLineMediaStream(*this, mediaFormat, sessionID, isSource, line);
@@ -592,13 +598,14 @@ PBoolean OpalLineConnection::OnOpenMediaStream(OpalMediaStream & mediaStream)
       patch->AddFilter(silenceDetector->GetReceiveHandler(), line.GetReadFormat());
   }
 
+  line.StopTone(); // In case a RoutingTone or RingTone is going
   return true;
 }
 
 
 PBoolean OpalLineConnection::SetAudioVolume(PBoolean source, unsigned percentage)
 {
-  PSafePtr<OpalLineMediaStream> stream = PSafePtrCast<OpalMediaStream, OpalLineMediaStream>(GetMediaStream(OpalMediaFormat::DefaultAudioSessionID, source));
+  PSafePtr<OpalLineMediaStream> stream = PSafePtrCast<OpalMediaStream, OpalLineMediaStream>(GetMediaStream(OpalMediaType::Audio(), source));
   if (stream == NULL)
     return false;
 
@@ -609,7 +616,7 @@ PBoolean OpalLineConnection::SetAudioVolume(PBoolean source, unsigned percentage
 
 unsigned OpalLineConnection::GetAudioSignalLevel(PBoolean source)
 {
-  PSafePtr<OpalLineMediaStream> stream = PSafePtrCast<OpalMediaStream, OpalLineMediaStream>(GetMediaStream(OpalMediaFormat::DefaultAudioSessionID, source));
+  PSafePtr<OpalLineMediaStream> stream = PSafePtrCast<OpalMediaStream, OpalLineMediaStream>(GetMediaStream(OpalMediaType::Audio(), source));
   if (stream == NULL)
     return UINT_MAX;
 
@@ -681,7 +688,8 @@ void OpalLineConnection::Monitor()
       else {
         // If we are in alerting state then we are B-Party
         OnConnectedInternal();
-        ownerCall.OpenSourceMediaStreams(*this, OpalMediaType::Audio());
+        if (GetMediaStream(OpalMediaType::Audio(), true) == NULL)
+          ownerCall.OpenSourceMediaStreams(*this, OpalMediaType::Audio());
       }
     }
   }
@@ -750,7 +758,7 @@ void OpalLineConnection::HandleIncoming(PThread &, INT)
       else {
         PTRACE(3, "LID Con\tDetected Caller ID \"" << callerId << '"');
         remotePartyNumber = words[0].Trim();
-        remotePartyName = words[1].Trim();
+        remotePartyName = words[2].Trim();
         if (remotePartyName.IsEmpty())
           remotePartyName = remotePartyNumber;
       }
@@ -758,6 +766,8 @@ void OpalLineConnection::HandleIncoming(PThread &, INT)
     else {
       PTRACE(3, "LID Con\tNo caller ID available.");
     }
+    if (remotePartyName.IsEmpty())
+      remotePartyName = "Unknown";
 
     // switch phase 
     SetPhase(AlertingPhase);
@@ -765,13 +775,20 @@ void OpalLineConnection::HandleIncoming(PThread &, INT)
   }
 
   if (!OnIncomingConnection(0, NULL)) {
+    PTRACE(3, "LID\tWaiting for RING to stop on " << *this);
+    while (line.GetRingCount() > 0) {
+      if (GetPhase() >= ReleasingPhase)
+        return;
+      PThread::Sleep(100);
+    }
     Release(EndedByCallerAbort);
     return;
   }
 
-  PTRACE(3, "LID\tRouted to \"" << ownerCall.GetPartyB() << "\" the "
+  PTRACE(3, "LID Con\tRouted to \"" << ownerCall.GetPartyB() << "\", "
          << (IsOriginating() ? "outgo" : "incom") << "ing connection " << *this);
-  ownerCall.OnSetUp(*this);
+  if (ownerCall.OnSetUp(*this) && line.IsTerminal() && GetPhase() < AlertingPhase)
+    line.PlayTone(OpalLineInterfaceDevice::RoutingTone);
 }
 
 
@@ -807,7 +824,8 @@ PBoolean OpalLineConnection::SetUpConnection()
 
     PTRACE(3, "LID Con\tNo remote party indicated, going off hook without dialing.");
     OnConnectedInternal();
-    ownerCall.OpenSourceMediaStreams(*this, OpalMediaType::Audio());
+    if (GetMediaStream(OpalMediaType::Audio(), true) == NULL)
+      ownerCall.OpenSourceMediaStreams(*this, OpalMediaType::Audio());
     return true;
   }
 
@@ -832,7 +850,8 @@ PBoolean OpalLineConnection::SetUpConnection()
 
   PTRACE(3, "LID Con\tGot ring back on " << line);
   // Start media before the OnAlerting to get progress tones (e.g. SIP 183 response)
-  ownerCall.OpenSourceMediaStreams(*this, OpalMediaType::Audio());
+  if (GetMediaStream(OpalMediaType::Audio(), true) == NULL)
+    ownerCall.OpenSourceMediaStreams(*this, OpalMediaType::Audio());
   SetPhase(AlertingPhase);
   OnAlerting();
 
@@ -853,11 +872,6 @@ PBoolean OpalLineConnection::SetUpConnection()
       return true;
     }
 
-    if (line.IsToneDetected() == OpalLineInterfaceDevice::BusyTone) {
-      Release(OpalConnection::EndedByRemoteBusy);
-      return false;
-    }
-
     PThread::Sleep(100);
   }
 
@@ -866,7 +880,7 @@ PBoolean OpalLineConnection::SetUpConnection()
          << ") on line " << line);
 
   if (m_dialParams.m_requireTones) {
-    Release(OpalConnection::EndedByRemoteBusy);
+    Release(OpalConnection::EndedByNoAnswer);
     return false;
   }
 
