@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21004 $
+ * $Revision: 21359 $
  * $Author: rjongbloed $
- * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
+ * $Date: 2008-10-16 04:29:02 +0000 (Thu, 16 Oct 2008) $
  */
 
 #include <ptlib.h>
@@ -120,7 +120,7 @@ bool SIPHandler::ShutDown()
     case Subscribed :
       SendRequest(Unsubscribing);
     case Unsubscribing :
-      return false;
+      return transactions.IsEmpty();
 
     default :
       break;
@@ -257,6 +257,7 @@ PBoolean SIPHandler::SendRequest(SIPHandler::State s)
   if (!retryLater) {
     // Restoring or first time, try every interface
     if (s == Restoring || transport->GetInterface().IsEmpty()) {
+      PWaitAndSignal mutex(transport->GetWriteMutex());
       if(transport->WriteConnect(WriteSIPHandler, this))
         return true;
     }
@@ -312,68 +313,50 @@ void SIPHandler::OnReceivedAuthenticationRequired(SIPTransaction & transaction,
                                                                                errorMsg);
   if (newAuth == NULL) {
     PTRACE(2, "SIP\t" << errorMsg);
-    OnFailed(SIP_PDU::Failure_UnAuthorised);
-    return;
-  }
-
-  // check to see if this is a follow-on from the last authentication scheme used
-  if (authentication != NULL && 
-      (authentication->GetClass() == newAuth->GetClass()) && 
-      newAuth->EquivalentTo(*authentication) &&
-      GetState() == SIPHandler::Subscribing) {
-    delete newAuth;
-    PTRACE(1, "SIP\tAuthentication already performed for " << proxyTrace << "Authentication Required");
-    OnFailed(SIP_PDU::Failure_UnAuthorised);
+    OnFailed(SIP_PDU::Failure_Forbidden);
     return;
   }
 
   // Try to find authentication parameters for the given realm,
   // if not, use the proxy authentication parameters (if any)
-  if (authenticationUsername.IsEmpty () && authenticationPassword.IsEmpty ()) {
-    if (endpoint.GetAuthentication(newAuth->GetAuthRealm(), authenticationAuthRealm, authenticationUsername, authenticationPassword)) {
-      PTRACE (3, "SIP\tFound auth info for realm " << newAuth->GetAuthRealm());
-    }
-    else {
-      SIPURL proxy = endpoint.GetProxy();
-      if (proxy.IsEmpty()) {
-        PTRACE (3, "SIP\tNo auth info for realm " << newAuth->GetAuthRealm());
-        delete newAuth;
-        OnFailed(SIP_PDU::Failure_UnAuthorised);
-        return;
-      }
-
+  PString authRealm = authenticationAuthRealm;
+  PString username = authenticationUsername;
+  PString password = authenticationPassword;
+  if (endpoint.GetAuthentication(newAuth->GetAuthRealm(), authRealm, username, password)) {
+    PTRACE (3, "SIP\tFound auth info for realm " << newAuth->GetAuthRealm());
+  }
+  else if (username.IsEmpty()) {
+    const SIPURL & proxy = endpoint.GetProxy();
+    if (!proxy.IsEmpty()) {
       PTRACE (3, "SIP\tNo auth info for realm " << newAuth->GetAuthRealm() << ", using proxy auth");
-      authenticationUsername = proxy.GetUserName();
-      authenticationPassword = proxy.GetPassword();
+      username = proxy.GetUserName();
+      password = proxy.GetPassword();
     }
   }
-  authenticationAuthRealm = newAuth->GetAuthRealm();
-  newAuth->SetUsername(authenticationUsername);
-  newAuth->SetPassword(authenticationPassword);
+
+  newAuth->SetUsername(username);
+  newAuth->SetPassword(password);
+
+  // check to see if this is a follow-on from the last authentication scheme used
+  if (GetState() == Subscribing && authentication != NULL && *newAuth == *authentication) {
+    delete newAuth;
+    PTRACE(1, "SIP\tAuthentication already performed using current credentials, not trying again.");
+    OnFailed(SIP_PDU::Failure_UnAuthorised);
+    return;
+  }
 
   // switch authentication schemes
   delete authentication;
   authentication = newAuth;
+  authenticationAuthRealm = newAuth->GetAuthRealm();
+  authenticationUsername = username;
+  authenticationPassword = password;
 
   // And end connect mode on the transport
   CollapseFork(transaction);
 
   // Restart the transaction with new authentication handler
-  SIPTransaction * newTransaction = CreateTransaction(*transport);
-  if (!authentication->Authorise(*newTransaction)) {
-    // don't send again if no authentication handler available
-    OnFailed(SIP_PDU::Failure_UnAuthorised);
-    delete newTransaction; // Not started yet, we need to delete
-    return;
-  }
-
-  // Section 8.1.3.5 of RFC3261 tells that the authenticated
-  // request SHOULD have the same value of the Call-ID, To and From.
-  newTransaction->GetMIME().SetFrom(transaction.GetMIME().GetFrom());
-  newTransaction->GetMIME().SetCallID(GetCallID());
-  if (!newTransaction->Start()) {
-    PTRACE(1, "SIP\tCould not restart REGISTER/SUBSCRIBE for Authentication Required");
-  }
+  SendRequest(GetState());
 }
 
 
@@ -478,6 +461,7 @@ SIPRegisterHandler::SIPRegisterHandler(SIPEndPoint & endpoint, const SIPRegister
                params.m_expire > 0 ? params.m_expire : endpoint.GetRegistrarTimeToLive().GetSeconds(),
                params.m_restoreTime, params.m_minRetryTime, params.m_maxRetryTime)
   , m_parameters(params)
+  , m_sequenceNumber(0)
 {
   m_parameters.m_expire = expire; // Put possibly adjusted value back
 
@@ -488,6 +472,9 @@ SIPRegisterHandler::SIPRegisterHandler(SIPEndPoint & endpoint, const SIPRegister
   authenticationUsername  = params.m_authID;
   authenticationPassword  = params.m_password;
   authenticationAuthRealm = params.m_realm;
+
+  if (authenticationUsername.IsEmpty())
+    authenticationUsername = targetAddress.GetUserName();
 }
 
 
@@ -499,22 +486,46 @@ SIPRegisterHandler::~SIPRegisterHandler()
 
 SIPTransaction * SIPRegisterHandler::CreateTransaction(OpalTransport & trans)
 {
-  m_parameters.m_expire = state != Unsubscribing ? expire : 0;
+  SIPRegister::Params params = m_parameters;
 
-  return new SIPRegister(endpoint, trans, GetRouteSet(), callID, m_parameters);
+  if (params.m_contactAddress.IsEmpty()) {
+    // Nothing explicit, put in all the bound interfaces.
+    unsigned qvalue = 1000;
+    PString userName = SIPURL(params.m_addressOfRecord).GetUserName();
+    OpalTransportAddressArray interfaces = endpoint.GetInterfaceAddresses(true, &trans);
+    for (PINDEX i = 0; i < interfaces.GetSize(); ++i) {
+      if (!params.m_contactAddress.IsEmpty())
+        params.m_contactAddress += ", ";
+      SIPURL contact(userName, interfaces[i]);
+      contact.Sanitise(SIPURL::ContactURI);
+      params.m_contactAddress += contact.AsQuotedString();
+      params.m_contactAddress.sprintf(qvalue < 1000 ? ";q=0.%03u" : ";q=1", qvalue);
+      qvalue -= 1000/interfaces.GetSize();
+    }
+  }
+  else {
+    // Sanitise the contact address URI provided
+    SIPURL contact(params.m_contactAddress);
+    contact.Sanitise(SIPURL::ContactURI);
+    params.m_contactAddress = contact.AsString();
+  }
+
+  params.m_expire = state != Unsubscribing ? expire : 0;
+
+  return new SIPRegister(endpoint, trans, GetRouteSet(), callID, m_sequenceNumber, params);
 }
 
 
 void SIPRegisterHandler::OnReceivedOK(SIPTransaction & transaction, SIP_PDU & response)
 {
-  std::vector<SIPURL> requestContacts, replyContacts;
+  std::list<SIPURL> requestContacts, replyContacts;
   transaction.GetMIME().GetContacts(requestContacts);
   response.GetMIME().GetContacts(replyContacts);
 
-  for (size_t request = 0; request < requestContacts.size(); request++) {
-    for (size_t reply = 0; reply < replyContacts.size(); reply++) {
-      if (requestContacts[request] == replyContacts[reply]) {
-        PString expires = SIPMIMEInfo::ExtractFieldParameter(replyContacts[reply].GetFieldParameters(), "expires");
+  for (std::list<SIPURL>::iterator request = requestContacts.begin(); request != requestContacts.end(); ++request) {
+    for (std::list<SIPURL>::iterator reply = replyContacts.begin(); reply != replyContacts.end(); ++reply) {
+      if (*request == *reply) {
+        PString expires = SIPMIMEInfo::ExtractFieldParameter(reply->GetFieldParameters(), "expires");
         if (expires.IsEmpty())
           SetExpire(response.GetMIME().GetExpires(endpoint.GetRegistrarTimeToLive().GetSeconds()));
         else
@@ -540,6 +551,7 @@ void SIPRegisterHandler::OnFailed(SIP_PDU::StatusCodes r)
 PBoolean SIPRegisterHandler::SendRequest(SIPHandler::State s)
 {
   SendStatus(SIP_PDU::Information_Trying);
+  m_sequenceNumber = endpoint.GetNextCSeq();
   return SIPHandler::SendRequest(s);
 }
 
@@ -826,6 +838,7 @@ PBoolean SIPSubscribeHandler::OnReceivedPresenceNOTIFY(SIP_PDU & request)
 {
   PString body = request.GetEntityBody();
   SIPURL from = request.GetMIME().GetFrom();
+  from.Sanitise(SIPURL::ExternalURI);
   PString basic;
   PString note;
 
@@ -836,8 +849,10 @@ PBoolean SIPSubscribeHandler::OnReceivedPresenceNOTIFY(SIP_PDU & request)
   PXMLElement *basicElement = NULL;
   PXMLElement *noteElement = NULL;
 
-  if (!xmlPresence.Load(body))
+  if (!xmlPresence.Load(body)) {
+    endpoint.OnPresenceInfoReceived (from.AsQuotedString(), basic, note);
     return PFalse;
+  }
 
   rootElement = xmlPresence.GetRootElement();
   if (rootElement == NULL)
@@ -866,7 +881,6 @@ PBoolean SIPSubscribeHandler::OnReceivedPresenceNOTIFY(SIP_PDU & request)
   if (noteElement)
     note = noteElement->GetData();
 
-  from.Sanitise(SIPURL::ExternalURI);
   endpoint.OnPresenceInfoReceived (from.AsQuotedString(), basic, note);
   return PTrue;
 }
