@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21004 $
+ * $Revision: 21283 $
  * $Author: rjongbloed $
- * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
+ * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
  */
 
 #include <ptlib.h>
@@ -34,10 +34,10 @@
 #pragma implementation "t120proto.h"
 #endif
 
-#include <opal/buildopts.h>
-
 #include <t120/t120proto.h>
 
+#if OPAL_T120DATA
+
 #include <h323/transaddr.h>
 #include <t120/x224.h>
 #include <asn/mcs.h>
@@ -227,4 +227,7 @@ PBoolean OpalT120Protocol::HandleDomain(const MCS_DomainMCSPDU & /*pdu*/)
 }
 
 
+#endif // OPAL_T120DATA
+
+
 /////////////////////////////////////////////////////////////////////////////
